/**
 * CRC - Canvas Render Components (alpha)
 *
 * MIT License
 *
 * Copyright 2020 Ben Lesh <ben@benlesh.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */ "use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    clearSharedState: ()=>clearSharedState,
    crc: ()=>crc,
    createElement: ()=>createElement,
    defineComp: ()=>defineComp,
    update: ()=>update,
    crcRef: ()=>crcRef,
    crcState: ()=>crcState,
    crcMemo: ()=>crcMemo,
    crcWhenChanged: ()=>crcWhenChanged,
    crcClick: ()=>crcClick,
    crcDblClick: ()=>crcDblClick,
    crcContextMenu: ()=>crcContextMenu,
    crcMouseMove: ()=>crcMouseMove,
    crcMouseOver: ()=>crcMouseOver,
    crcMouseOut: ()=>crcMouseOut,
    crcCursor: ()=>crcCursor,
    crcRectPath: ()=>crcRectPath,
    crcLinePath: ()=>crcLinePath,
    crcSvgPath: ()=>crcSvgPath,
    path: ()=>path,
    rect: ()=>rect,
    line: ()=>line,
    verticalLine: ()=>verticalLine,
    horizontalLine: ()=>horizontalLine,
    img: ()=>img,
    svgPath: ()=>svgPath,
    Text: ()=>Text,
    text: ()=>text,
    g: ()=>g
});
////////////////////////////////////////////////////////////////////////////////////////
// Shared State
////////////////////////////////////////////////////////////////////////////////////////
let crcInstances = new Map();
let _currentCRCInstance = undefined;
let _componentRefs = undefined;
let _canvas = undefined;
let _componentIsMounting = false;
let _compnentHookIndex = 0;
let _renderContext = undefined;
let _unseenIds = undefined;
let anonElementNames = new WeakMap();
let anon = 0;
/**
 * Used to get a name for an anonymous function.
 * @param fn The anonymous function to get a name for
 * @returns A name for the anonymous function
 */ function getAnonElemName(fn) {
    if (!anonElementNames.has(fn)) {
        anonElementNames.set(fn, `Anon${anon++}`);
    }
    return anonElementNames.get(fn);
}
function clearSharedState() {
    for (const crcInstance of crcInstances.values()){
        cleanupCRCInstace(crcInstance);
    }
    crcInstances.clear();
    anonElementNames = new WeakMap();
    clearTempState();
}
function cleanupCRCInstace(crcInstance) {
    crcInstance.abortController.abort();
}
function clearTempState() {
    _unseenIds = undefined;
    _canvas = undefined;
    _renderContext = undefined;
    clearComponentState();
}
function clearComponentState() {
    _componentRefs = undefined;
    _componentIsMounting = false;
    _compnentHookIndex = 0;
}
function crc(canvas, element, config) {
    if (!crcInstances.has(canvas)) {
        const ac = new AbortController();
        const signal = config?.signal;
        const cleanup = ()=>{
            crcInstances.delete(canvas);
            ac.abort();
        };
        if (signal) {
            config.signal.addEventListener('abort', cleanup, {
                once: true,
                signal: ac.signal
            });
        }
        const crcInstance = {
            root: element,
            refs: new Map(),
            animationFrameId: 0,
            renderTimestamp: 0,
            abortController: ac
        };
        detectElementRemoval(canvas, cleanup, signal);
        crcInstances.set(canvas, crcInstance);
    }
    update(canvas);
}
function createElement(comp, props) {
    return {
        type: comp,
        props
    };
}
function defineComp(compFn) {
    return (props)=>createElement(compFn, props);
}
/**
 * Gets the root element for a canvas and renders it. This is the
 * entry point for rendering the entire tree.
 * @param canvas The canvas to render on
 */ function executeRender(canvas, renderTimestamp) {
    const crcInstance = crcInstances.get(canvas);
    crcInstance.animationFrameId = 0;
    if (crcInstance.abortController.signal.aborted) return;
    try {
        _currentCRCInstance = crcInstance;
        _currentCRCInstance.renderTimestamp = renderTimestamp;
        _canvas = canvas;
        const rootElement = _currentCRCInstance.root;
        const ctx = get2dContext(canvas);
        _renderContext = ctx;
        _unseenIds = new Set(_currentCRCInstance.refs.keys());
        const id = 'root';
        _unseenIds.delete(id);
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.drawImage(render(rootElement, id, undefined), 0, 0);
        for (const id1 of _unseenIds){
            // HACK: I want to be more efficient here, but this is fine for now
            _currentCRCInstance.refs.get(id1)?.abortController.abort();
            _currentCRCInstance.refs.delete(id1);
        }
    } finally{
        clearTempState();
    }
}
function markDirty(compRefs) {
    while(compRefs){
        compRefs.isDirty = true;
        compRefs = compRefs.parent;
    }
}
function update(canvas) {
    const crcInstance = crcInstances.get(canvas);
    if (!crcInstance) {
        throw new Error('Canvas does not have CRC mounted.');
    }
    if (!crcInstance.animationFrameId) {
        crcInstance.animationFrameId = requestAnimationFrame((ts)=>executeRender(canvas, ts));
    }
}
function getElementTypeName(element) {
    return element.props.key || element.type.name || getAnonElemName(element.type);
}
function render(element, parentId, parent) {
    try {
        const id = parentId + ':' + getElementTypeName(element);
        _unseenIds.delete(id);
        const { width , height  } = _canvas;
        _componentIsMounting = !_currentCRCInstance.refs.has(id);
        if (_componentIsMounting) {
            _currentCRCInstance.refs.set(id, {
                id,
                parent,
                element,
                refs: [],
                abortController: new AbortController(),
                offscreenCanvas: new OffscreenCanvas(width, height),
                isDirty: true
            });
        }
        _componentRefs = _currentCRCInstance.refs.get(id);
        _compnentHookIndex = 0;
        const { offscreenCanvas  } = _componentRefs;
        if (_componentIsMounting || _componentRefs.isDirty || offscreenCanvas.width !== width || offscreenCanvas.height !== height || !shallowEqual(_componentRefs.element.props, element.props)) {
            _componentRefs.element = element;
            offscreenCanvas.width = width;
            offscreenCanvas.height = height;
            const ctx = offscreenCanvas.getContext('2d');
            const result = _componentRefs.element.type(_componentRefs.element.props, ctx);
            _componentRefs.isDirty = false;
            if (result) {
                if (Array.isArray(result)) {
                    for(let i = 0; i < result.length; i++){
                        const child = result[i];
                        ctx.drawImage(render(child, id + ':' + i, _componentRefs), 0, 0);
                    }
                } else {
                    ctx.drawImage(render(result, id, _componentRefs), 0, 0);
                }
            }
        } else {
            return offscreenCanvas;
        }
        return offscreenCanvas;
    } finally{
        clearComponentState();
    }
}
function crcRef(init) {
    if (_componentIsMounting) {
        _componentRefs.refs.push({
            type: 'ref',
            current: init
        });
    }
    return _componentRefs.refs[_compnentHookIndex++];
}
function crcState(init) {
    if (_componentIsMounting) {
        const ref = {
            type: 'state',
            value: init,
            setter
        };
        const canvas = _canvas;
        const compRefs = _componentRefs;
        function setter(newValueOrFactory) {
            ref.value = typeof newValueOrFactory === 'function' ? newValueOrFactory(ref.value) : newValueOrFactory;
            markDirty(compRefs);
            update(canvas);
        }
        _componentRefs.refs.push(ref);
    }
    const ref1 = _componentRefs.refs[_compnentHookIndex++];
    return [
        ref1.value,
        ref1.setter
    ];
}
function crcMemo(create, deps) {
    const ref = crcRef();
    // NOTE: sync updates
    crcWhenChanged(()=>{
        ref.current = create();
    }, deps);
    return ref.current;
}
function crcWhenChanged(callback, deps) {
    if (_componentIsMounting) {
        const ref = {
            type: 'whenChanged',
            lastDeps: undefined,
            teardown: undefined
        };
        _componentRefs.refs.push(ref);
    }
    const hookIndex = _compnentHookIndex++;
    const ref1 = _componentRefs.refs[hookIndex];
    if (!ref1.lastDeps || !deps || !shallowArrayEquals(deps, ref1.lastDeps)) {
        const lastTeardown = ref1.teardown;
        const signal = _currentCRCInstance.abortController.signal;
        if (lastTeardown) {
            signal.removeEventListener('abort', lastTeardown);
            _componentRefs.abortController.signal.removeEventListener('abort', lastTeardown);
            lastTeardown();
        }
        ref1.lastDeps = deps;
        const newTeardown = callback();
        if (newTeardown) {
            ref1.teardown = newTeardown;
            signal.addEventListener('abort', newTeardown, {
                once: true
            });
            _componentRefs.abortController.signal.addEventListener('abort', newTeardown, {
                once: true
            });
        }
    }
}
function isHit({ canvas , transform , path , x , y , fill , lineInteractionWidth  }) {
    const ctx = get2dContext(canvas);
    ctx.setTransform(transform);
    if (fill && ctx.isPointInPath(path, x, y)) {
        return true;
    }
    if (lineInteractionWidth > 0) {
        ctx.lineWidth = lineInteractionWidth;
        if (ctx.isPointInStroke(path, x, y)) {
            return true;
        }
    }
    return false;
}
function crcBasicMouseEvent({ type , path , handler , fill =false , lineInteractionWidth =0  }) {
    crcWhenChanged(()=>{
        const ac = new AbortController();
        const transform = _renderContext.getTransform();
        _canvas.addEventListener(type, (e)=>{
            const canvas = e.target;
            const [x, y] = getMouseCoordinates(e);
            if (isHit({
                canvas,
                path,
                x,
                y,
                fill,
                transform,
                lineInteractionWidth
            })) {
                handler({
                    x,
                    y,
                    originalEvent: e
                });
            }
        }, {
            signal: ac.signal
        });
        return ()=>{
            ac.abort();
        };
    }, [
        type,
        path,
        handler,
        fill,
        lineInteractionWidth
    ]);
}
function crcClick(config) {
    crcBasicMouseEvent({
        ...config,
        type: 'click'
    });
}
function crcDblClick(config) {
    crcBasicMouseEvent({
        ...config,
        type: 'dblclick'
    });
}
function crcContextMenu(config) {
    crcBasicMouseEvent({
        ...config,
        type: 'contextmenu'
    });
}
function crcMouseMove(config) {
    crcBasicMouseEvent({
        ...config,
        type: 'mousemove'
    });
}
function crcMouseOver(config) {
    crcMouseOverOrOut({
        ...config,
        type: 'mouseover'
    });
}
function crcMouseOut(config) {
    crcMouseOverOrOut({
        ...config,
        type: 'mouseout'
    });
}
function crcMouseOverOrOut({ type , path , handler , fill =false , lineInteractionWidth =0  }) {
    const overRef = crcRef(false);
    crcWhenChanged(()=>{
        const ac = new AbortController();
        const transform = _renderContext.getTransform();
        _canvas.addEventListener('mousemove', (e)=>{
            const canvas = e.target;
            const [x, y] = getMouseCoordinates(e);
            const isOver = isHit({
                canvas,
                transform,
                path,
                x,
                y,
                fill,
                lineInteractionWidth
            });
            const wasOver = overRef.current;
            if (isOver) {
                overRef.current = true;
                if (type === 'mouseover' && !wasOver) {
                    handler({
                        x,
                        y,
                        originalEvent: e
                    });
                }
            } else {
                overRef.current = false;
                if (type === 'mouseout' && wasOver) {
                    handler({
                        x,
                        y,
                        originalEvent: e
                    });
                }
            }
        }, {
            signal: ac.signal
        });
        _canvas.addEventListener('mouseout', (e)=>{
            overRef.current = false;
            if (type === 'mouseout') {
                const [x, y] = getMouseCoordinates(e);
                handler({
                    x,
                    y,
                    originalEvent: e
                });
            }
        }, {
            signal: ac.signal
        });
        return ()=>ac.abort();
    }, [
        type,
        path,
        handler,
        fill,
        lineInteractionWidth
    ]);
}
function crcCursor({ path , style , fill =false , lineInteractionWidth =0  }) {
    crcMouseOver({
        path,
        handler: (e)=>{
            const canvas = e.originalEvent.target;
            canvas.style.cursor = style;
        },
        fill,
        lineInteractionWidth
    });
    crcMouseOut({
        path,
        handler: (e)=>{
            const canvas = e.originalEvent.target;
            canvas.style.removeProperty('cursor');
        },
        fill,
        lineInteractionWidth
    });
}
function crcRectPath(x, y, width, height) {
    return crcMemo(()=>{
        const path = new Path2D();
        path.rect(x, y, width, height);
        return path;
    }, [
        x,
        y,
        width,
        height
    ]);
}
function crcLinePath(coords, config) {
    return crcMemo(()=>{
        const path = new Path2D();
        for(let i = 0; i < coords.length; i++){
            const [x, y] = coords[i];
            if (i === 0) {
                path.moveTo(x, y);
            } else {
                path.lineTo(x, y);
            }
        }
        if (config?.closePath) {
            path.closePath();
        }
        return path;
    }, coords.flat());
}
function crcSvgPath(svgPathData) {
    return crcMemo(()=>new Path2D(svgPathData), [
        svgPathData
    ]);
}
function Path(props, ctx) {
    const { alpha , path , fillStyle , strokeStyle , lineWidth , lineInteractionWidth =lineWidth  } = props;
    if (alpha) {
        ctx.globalAlpha = ctx.globalAlpha * alpha;
    }
    if (fillStyle) {
        ctx.fillStyle = fillStyle;
        ctx.fill(path);
    }
    if (strokeStyle && lineWidth && lineWidth > 0) {
        ctx.strokeStyle = strokeStyle, ctx.lineWidth = lineWidth;
        ctx.stroke(path);
    }
    const fill = !!fillStyle;
    wireCommonEvents(props, path, fill, lineInteractionWidth);
}
const path = defineComp(Path);
function wireCommonEvents(props, path, fill, lineInteractionWidth) {
    const { onClick , onContextMenu , onDblClick , onMouseMove , onMouseOut , onMouseOver  } = props;
    if (onClick) {
        crcClick({
            path,
            handler: onClick,
            fill,
            lineInteractionWidth
        });
    }
    if (onContextMenu) {
        crcContextMenu({
            path,
            handler: onContextMenu,
            fill,
            lineInteractionWidth
        });
    }
    if (onDblClick) {
        crcDblClick({
            path,
            handler: onDblClick,
            fill,
            lineInteractionWidth
        });
    }
    if (onMouseOver) {
        crcMouseOver({
            path,
            handler: onMouseOver,
            fill,
            lineInteractionWidth
        });
    }
    if (onMouseMove) {
        crcMouseMove({
            path,
            handler: onMouseMove,
            fill,
            lineInteractionWidth
        });
    }
    if (onMouseOut) {
        crcMouseOut({
            path,
            handler: onMouseOut,
            fill,
            lineInteractionWidth
        });
    }
    const { cursor  } = props;
    if (cursor) {
        crcCursor({
            path,
            style: cursor,
            fill,
            lineInteractionWidth
        });
    }
}
function Rect(props, ctx) {
    const { x , y , width , height , ...pathProps } = props;
    const rectPath = crcRectPath(x, y, width, height);
    return Path({
        ...pathProps,
        path: rectPath
    }, ctx);
}
const rect = defineComp(Rect);
function Line(props, ctx) {
    const { coords , ...pathProps } = props;
    const linePath = crcLinePath(coords);
    return Path({
        ...pathProps,
        path: linePath
    }, ctx);
}
const line = defineComp(Line);
function VerticalLine(props, ctx) {
    const { x: initialX , top =0 , bottom =ctx.canvas.height , alignToPixelGrid =false , ...lineProps } = props;
    const x = alignToPixelGrid ? Math.round(initialX) - calculatePixelGridOffset(props.lineWidth) : initialX;
    const coords = [
        [
            x,
            top
        ],
        [
            x,
            bottom
        ]
    ];
    return Line({
        coords,
        ...lineProps
    }, ctx);
}
const verticalLine = defineComp(VerticalLine);
function HorizontalLine(props, ctx) {
    const { y: initialY , left =0 , right =ctx.canvas.width , alignToPixelGrid =false , ...lineProps } = props;
    const y = alignToPixelGrid ? Math.round(initialY) - calculatePixelGridOffset(props.lineWidth) : initialY;
    const coords = [
        [
            left,
            y
        ],
        [
            right,
            y
        ]
    ];
    return Line({
        coords,
        ...lineProps
    }, ctx);
}
const horizontalLine = defineComp(HorizontalLine);
function Img(props, ctx) {
    let { alpha , src , x , y , width , height , lineWidth , strokeStyle , ...otherProps } = props;
    if (alpha) {
        ctx.globalAlpha = ctx.globalAlpha * alpha;
    }
    const [image, setImage] = crcState(null);
    crcWhenChanged(()=>{
        const img = new Image();
        img.onload = ()=>{
            setImage(img);
        };
        img.src = src;
    }, [
        src
    ]);
    width ??= image?.width ?? 0;
    height ??= image?.height ?? 0;
    const imagePath = crcRectPath(x, y, width, height);
    crcCursor({
        path: imagePath,
        style: props.cursor,
        fill: true,
        lineInteractionWidth: props.lineWidth
    });
    if (image) {
        ctx.drawImage(image, x, y, width, height);
    }
    if (strokeStyle && lineWidth) {
        ctx.strokeStyle = strokeStyle;
        ctx.lineWidth = lineWidth;
        ctx.stroke(imagePath);
    }
    wireCommonEvents(otherProps, imagePath, true, lineWidth);
}
const img = defineComp(Img);
function SvgPath(props, ctx) {
    const { d , ...pathProps } = props;
    const svgPath = crcSvgPath(d);
    return Path({
        ...pathProps,
        path: svgPath
    }, ctx);
}
const svgPath = defineComp(SvgPath);
function G(props, ctx) {
    let transform = new DOMMatrix();
    const { scaleX =1 , scaleY =1  } = props;
    if (scaleX !== 1 || scaleY !== 1) {
        transform = transform.scale(scaleX, scaleY);
    }
    const { rotate  } = props;
    if (rotate) {
        const [rox, roy] = props.rotateOrigin ?? [
            0,
            0
        ];
        transform = transform.translate(rox, roy).rotate(rotate).translate(-rox, -roy);
    }
    const { x =0 , y =0  } = props;
    if (x || y) {
        transform = transform.translate(x, y);
    }
    const { skewX  } = props;
    if (skewX) {
        transform = transform.skewX(skewX);
    }
    const { skewY  } = props;
    if (skewY) {
        transform = transform.skewY(skewY);
    }
    ctx.setTransform(transform);
    return props.children;
}
function Text(props, ctx) {
    const { font ='13px sans-serif' , textBaseline ='top' , textAlign ='left'  } = props;
    ctx.font = font;
    ctx.textBaseline = textBaseline;
    ctx.textAlign = textAlign;
    const { strokeStyle , lineWidth , fillStyle , x , maxWidth  } = props;
    const renderText = (txt, y)=>{
        if (strokeStyle && lineWidth) {
            ctx.strokeStyle = strokeStyle;
            ctx.lineWidth = lineWidth;
            ctx.strokeText(txt, x, y, maxWidth);
        }
        if (fillStyle) {
            ctx.fillStyle = fillStyle;
            ctx.fillText(txt, x, y, maxWidth);
        }
    };
    let textWidth = 0;
    let textHeight = 0;
    const { y , wordWrap  } = props;
    if (wordWrap) {
        const words = props.text.split(' ');
        let line = '';
        let lineCount = 0;
        const updateTextBounds = (line)=>{
            const bounds = ctx.measureText(line);
            textWidth = Math.max(bounds.width);
            textHeight = lineCount * lineHeight + bounds.actualBoundingBoxDescent - bounds.actualBoundingBoxAscent;
        };
        const { maxHeight , lineHeight =13 , overflow ='clip'  } = props;
        while(words.length){
            const currentYOffset = lineCount * lineHeight;
            const nextYOffset = currentYOffset + lineHeight;
            if (maxHeight < nextYOffset + lineHeight) {
                const remaining = words.join(' ');
                const lastLine = overflow === 'ellipsis' && maxWidth ? getEllipsisText(ctx, remaining, maxWidth) : remaining;
                updateTextBounds(lastLine);
                renderText(lastLine, y + currentYOffset);
                break;
            } else {
                if (maxWidth < ctx.measureText(line + words[0]).width) {
                    updateTextBounds(line);
                    renderText(line, y + currentYOffset);
                    line = '';
                    lineCount++;
                }
                line += words.shift() + ' ';
            }
        }
    } else {
        const text = props.overflow === 'ellipsis' && maxWidth ? getEllipsisText(ctx, props.text, maxWidth) : props.text;
        renderText(text, y);
        const bounds = ctx.measureText(text);
        textWidth = bounds.width;
        textHeight = bounds.actualBoundingBoxDescent - bounds.actualBoundingBoxAscent;
    }
    const textPath = crcRectPath(x, y, textWidth, textHeight);
    wireCommonEvents(props, textPath, true, lineWidth);
}
const text = defineComp(Text);
function getEllipsisText(renderContext, text, maxWidth) {
    const metrics = renderContext.measureText(text);
    if (metrics.width < maxWidth) {
        return text;
    }
    let low = -1;
    let high = text.length;
    while(1 + low < high){
        const mid = low + (high - low >> 1);
        if (isTextTooWide(renderContext, text.substring(0, mid) + '...', maxWidth)) {
            high = mid;
        } else {
            low = mid;
        }
    }
    const properLength = high - 1;
    return text.substring(0, properLength) + '...';
}
function isTextTooWide(renderContext, text, maxWidth) {
    const metrics = renderContext.measureText(text);
    return maxWidth < metrics.width;
}
const g = defineComp(G);
////////////////////////////////////////////////////////////////////////////////////////
// Utility functions
////////////////////////////////////////////////////////////////////////////////////////
function getMouseCoordinates(event) {
    const target = event.target;
    const bounds = target.getBoundingClientRect();
    const x = event.clientX - bounds.left;
    const y = event.clientY - bounds.top;
    return [
        x,
        y
    ];
}
function shallowArrayEquals(arr1, arr2) {
    if (arr1.length !== arr2.length) {
        return false;
    }
    for(let i = 0; i < arr1.length; i++){
        if (arr1[i] !== arr2[i]) {
            return false;
        }
    }
    return true;
}
function get2dContext(canvas) {
    const ctx = canvas.getContext('2d');
    if (!ctx) {
        throw new Error('Unable to get 2d context!');
    }
    return ctx;
}
function calculatePixelGridOffset(lineWidth) {
    return (lineWidth ?? 0) / 2 % 1;
}
function detectElementRemoval(element, handler, signal) {
    const observer = new MutationObserver(()=>{
        if (!document.body.contains(element)) {
            observer.disconnect();
            // @ts-ignore
            element.__crcRemoved = true;
        }
    });
    observer.observe(document.body, {
        childList: true,
        subtree: true
    });
    signal?.addEventListener('abort', ()=>{
        observer.disconnect();
    });
}
function shallowEqual(obj1, obj2) {
    const obj1Keys = Object.keys(obj1);
    const obj2Keys = Object.keys(obj2);
    if (obj1Keys.length !== obj2Keys.length) {
        return false;
    }
    for (const key of obj1Keys){
        if (obj1[key] !== obj2[key]) {
            return false;
        }
    }
    return true;
}

//# sourceMappingURL=index.js.map